from cmdcheatsheet.commands.core import delete_command
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class DeleteCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--delete', '-d'],
            'Delete a <command> by id.',
            [CommandArgument('id')])

    def handler(self, args):
        delete_command(int(args[0]))



from cmdcheatsheet.config.display_available_configs import DisplayAvailableConfigs
from cmdcheatsheet.config.display_config import DisplayConfig
from cmdcheatsheet.config.remove_config import RemoveConfig
from cmdcheatsheet.config.set_config import SetConfig
from cmdcheatsheet.config.set_config_to_default import SetConfigToDefault
from cmdcheatsheet.config.set_single_config_to_default import SetSingleConfigToDefault

config_actions = [
    DisplayAvailableConfigs(),
    DisplayConfig(),
    RemoveConfig(),
    SetConfig(),
    SetConfigToDefault(),
    SetSingleConfigToDefault()
]



from rich.prompt import Confirm
from cmdcheatsheet.config.core import set_config_to_default
from cmdcheatsheet.shared.models import CommandDetails


class SetConfigToDefault(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--set-config-to-default', '-sctd'],
            'Set the configuration to default.')

    def handler(self, _):
        is_yes = Confirm.ask(
            "All config changes will be lost. \n" +
            "Are you sure you want to set your config to default?")
        if is_yes:
            set_config_to_default()



from cmdcheatsheet.commands.display import display_command_name_list
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class AvailableCommandNames(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--available-command-names', '-acn'],
            'Show all stored command names.',
            [CommandArgument('number_of_columns', False)])

    def handler(self, args):
        display_command_name_list(int(args[0]) if len(args) > 0 else 5)



import os
from cmdcheatsheet.config.models import DefaultConfigurations, DefaultConfig

USER_HOME_DIR = os.path.expanduser('~')

CONFIG_DIR = f"{USER_HOME_DIR}/.config/cmdcheatsheet"
CONFIG_NAME = "config.json"
CONFIG_LOCATION = f"{CONFIG_DIR}/{CONFIG_NAME}"

STORE_NAME = "commands.json"
DEFAULT_COMMANDS_STORE_LOCATION = f"{CONFIG_DIR}/{STORE_NAME}"

# Config property names
CURR_STORE_LOCATION_CONF = 'currentStoreLocation'
ALT_STORES_CONF = 'alternativeStores'
ALT_STORE_NAME_CONF = 'storeName'
ALT_STORE_LOCATION_CONF = 'storeLocation'

DEFAULT_CONFIG = DefaultConfigurations([
    DefaultConfig(
        CURR_STORE_LOCATION_CONF,
        DEFAULT_COMMANDS_STORE_LOCATION,
        'Path to file in JSON format that consists of the command list.'
    ),
    DefaultConfig(
        ALT_STORES_CONF,
        [],
        'Path list to JSON files that might be used as an alternative commands store location.'
    )
])




# from cmdcheatsheet.alt_store import * 
from cmdcheatsheet.alt_store.add_alt_store import AddAltStore
from cmdcheatsheet.alt_store.delete_alt_store import DeleteAltStore
from cmdcheatsheet.alt_store.display_applied_alt_store_name import DisplayAppliedAltStoreName
from cmdcheatsheet.alt_store.display_available_alt_stores import DisplayAvailableAltStores
from cmdcheatsheet.alt_store.switch_to_alt_store import SwitchToAltStore
from cmdcheatsheet.alt_store.update_alt_store import UpdateAltStore

alt_store_actions = [
    AddAltStore(),
    DeleteAltStore(),
    DisplayAppliedAltStoreName(),
    DisplayAvailableAltStores(),
    SwitchToAltStore(),
    UpdateAltStore()
]



from cmdcheatsheet.config.core import read_config, set_config_value
from cmdcheatsheet.shared.json_file import write_json
from cmdcheatsheet.config.consts import *
from cmdcheatsheet.alt_store.display import display_alt_store_list


def add_alt_store(alt_store):
    config = read_config()
    alt_stores = config.get(ALT_STORES_CONF)
    alt_store_to_add = dict()
    alt_store_to_add[ALT_STORE_NAME_CONF] = alt_store.name
    alt_store_to_add[ALT_STORE_LOCATION_CONF] = alt_store.location
    alt_stores.append(alt_store_to_add)
    write_json(CONFIG_LOCATION, config)
    
def update_alt_store(alt_store_to_update):
    config = read_config()
    for alt_store in config.get(ALT_STORES_CONF) :
        if alt_store.get(ALT_STORE_NAME_CONF) == alt_store_to_update.name:
            alt_store[ALT_STORE_LOCATION_CONF] = alt_store_to_update.location
    write_json(CONFIG_LOCATION, config)

def delete_alt_store(store_name):
    config = read_config()
    alt_stores = config.get(ALT_STORES_CONF)
    config[ALT_STORES_CONF] = exclude_by_name_filter(alt_stores, store_name)
    write_json(CONFIG_LOCATION, config)

def is_existing_store_name(store_name):
    config = read_config()
    alt_stores = config.get(ALT_STORES_CONF)
    alt_store_names = [store.get(ALT_STORE_NAME_CONF) for store in alt_stores]
    return store_name in alt_store_names

def exclude_by_name_filter(alt_stores, store_name):
    return [store for store in alt_stores if store.get(ALT_STORE_NAME_CONF) != store_name]

def display_alt_stores():
    display_alt_store_list(read_config().get(ALT_STORES_CONF))

def find_alt_store_by_name(store_name):
    config = read_config()
    alt_stores = config.get(ALT_STORES_CONF)
    return next((s for s in alt_stores if s.get(ALT_STORE_NAME_CONF) == store_name), None)

def switch_to_alt_store(store_name):
    alt_store = find_alt_store_by_name(store_name)
    store_location = alt_store.get(ALT_STORE_LOCATION_CONF)
    set_config_value(CURR_STORE_LOCATION_CONF, store_location)

def get_applied_alt_store_name():
    alt_store_name = None
    config = read_config()
    commands_store_location = config.get(CURR_STORE_LOCATION_CONF)
    alt_stores = config.get(ALT_STORES_CONF)
    for store in alt_stores:
        if store.get(ALT_STORE_LOCATION_CONF) == commands_store_location:
            alt_store_name = store.get(ALT_STORE_NAME_CONF)
    return alt_store_name if alt_store_name else ''




from cmdcheatsheet.config.display import display_available_configs
from cmdcheatsheet.shared.models import CommandDetails


class DisplayAvailableConfigs(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--display-available-configs', '-dac'],
            'Display available configurations.')

    def handler(self, _):
        display_available_configs()



from cmdcheatsheet.alt_store.core import display_alt_stores
from cmdcheatsheet.shared.models import CommandDetails


class DisplayAvailableAltStores(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--display-available-alternative-stores', '-daas'],
        "Display available alternative stores.")

    def handler(self, _):
        display_alt_stores()



from rich.prompt import Confirm
from cmdcheatsheet.config.core import set_single_config_value_to_default
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class SetSingleConfigToDefault(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--set-single-config-to-default', '-ssctd'],
        'Set a single configuration to default.',
        [CommandArgument('key')])

    def handler(self, args):
        key = args[0]
        is_yes = Confirm.ask(f"Are you sure you want to set '{key}' to default?")
        if is_yes:
            set_single_config_value_to_default(key)




from cmdcheatsheet.commands.display import display_command_by_name
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class FindCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--find', '-f'],
            'Search for a command.',
            [CommandArgument('command')])

    def handler(self, args):
        display_command_by_name(args[0])





from cmdcheatsheet.general.consts import HELP_RESPONSIBLE_COMMANDS
from cmdcheatsheet.general.core import help as program_help
from cmdcheatsheet.shared.models import CommandDetails


class Help(CommandDetails):
    def __init__(self):
       super().__init__(HELP_RESPONSIBLE_COMMANDS, 'Show a program help notes.')

    def handler(self, args):
        program_help(args[0])




from cmdcheatsheet.alt_store.core import delete_alt_store, is_existing_store_name
from cmdcheatsheet.alt_store.messages import show_store_with_name_not_exists
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class DeleteAltStore(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--delete-alternative-store', '-das'] ,
        "Delete alternative commands store (JSON file).",
        [CommandArgument('store_name')])

    def handler(self, args):
        store_name = args[0]
        if is_existing_store_name(store_name):
            delete_alt_store(store_name)
        else: 
            show_store_with_name_not_exists(store_name)



import json
import os
from dataclasses import asdict
from cmdcheatsheet.config.core import get_store_location
from cmdcheatsheet.config.consts import DEFAULT_COMMANDS_STORE_LOCATION
from cmdcheatsheet.shared.json_file import write_json
from cmdcheatsheet.shared.display import display_info, display_error
from cmdcheatsheet.shared.models import Command


def command_to_name(command):
    command_split = command.split(' ')
    command_name = command_split[1] if command_split[0] == 'sudo' else command_split[0]
    if command_name.endswith(','):
        command_name = command_name.replace(',', '')
    return command_name

def get_command_name_list():
    commands = get_commands()
    command_names = []
    for command in commands:
        command_name = command_to_name(command.command)
        if command_name not in command_names:
            command_names.append(command_name)
    return command_names

def group_commands_by_name():
    commands = get_commands()
    command_dict = {}
    for command in commands:
        command_name = command_to_name(command.command)
        if command_dict.get(command_name) is None:
            command_dict[command_name] = [command]
        else:
            command_dict.get(command_name).append(command)
    return command_dict

def add_command(command): 
    commands = get_commands()
    command_to_add = find_command_by_name(commands, command.command)
    if command_to_add is None:
        command.id = get_index()
        commands.append(command)
        save_commands(commands)
        display_info(f"Command '{command.command}' added.")
        return
    display_info(f"Command '{command.command}' already exists.")

def delete_command(command_id):
    commands = get_commands()
    command_to_delete = find_command_by_id(commands, command_id)
    if command_to_delete is not None:
        commands.remove(command_to_delete)
        save_commands(commands)
        display_info(f"Command with id: {command_id} removed.")
        return
    display_error(f"Command with id: {command_id} not found.")

def update_command(command):
    commands = get_commands()
    command_to_update = find_command_by_id(commands, command.id)
    if command_to_update is not None:
        command_to_update.command = command.command
        command_to_update.description = command.description
        save_commands(commands)
        display_info(f"Command with id: {command.id} updated.")
        return
    display_error(f"Command with id: {command.id} not found.")

def find_command_by_name(commands, command_value):
    return next((c for c in commands if c.command == command_value), None)


def find_command_by_id(commands, id):
    return next((c for c in commands if c.id == id), None)

def get_commands():
    with open(get_store_location()) as f:
      commands = json.load(f)
    return [Command(c.get('command'), c.get('description'), c.get('id')) for c in commands]

def get_index():
    commands = get_commands()
    if not commands:
        return 1
    else:
        last_command = commands[-1]
        return last_command.id + 1

def get_commands_by_includes_command_name(command_name):
    commands = []
    for command in get_commands():
        command_split = command.command.replace(',', '').split(' ')
        if command_name in command_split:
            commands.append(command)
    return commands

def save_commands(commands):
    commands_to_save = [asdict(c) for c in commands]
    write_json(get_store_location(), commands_to_save)

def setup_commands_store_config():
    if not os.path.exists(DEFAULT_COMMANDS_STORE_LOCATION):
        write_json(get_store_location(), [])




from cmdcheatsheet.config.core import set_config_value
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails
from cmdcheatsheet.config.validators import validate_configuration


class SetConfig(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--set-config', '-sc'],
            'Set config.',
            [CommandArgument('key'), CommandArgument('value')])

    def handler(self, args):
        key = args[0]
        value = args[1]
        if validate_configuration(key, value):
            set_config_value(key=key, value=value)



from cmdcheatsheet.config.display import display_config
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class DisplayConfig(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--display-configs', '-dc'],
            'Display configurations.',
            [CommandArgument('key', False)])

    def handler(self, args):
        if args:
            display_config(args[0])
        else:
            display_config()



from cmdcheatsheet.shared.consts import VERSION
from cmdcheatsheet.general.display import display_version
from cmdcheatsheet.shared.models import CommandDetails


class Version(CommandDetails):
    def __init__(self):
       super().__init__(['--version', '-v'], 'Display version.')

    def handler(self, _):
        display_version(VERSION)



from rich import print
from cmdcheatsheet.config.core import read_config
from cmdcheatsheet.config.consts import DEFAULT_CONFIG
from cmdcheatsheet.shared.display import BLUE, GREEN, display_error

def display_config(key=''): 
    config = read_config()
    if key:
        if key in DEFAULT_CONFIG.configuration_keys():
            display_configs(key, config[key])
        else:
            display_error(f"No configuration found with the name {key}")
    else:
        for key in config:
            display_configs(key, config[key])

def display_available_configs():
    for config in DEFAULT_CONFIG.configs:
        display_configs(config.key, config.desc)

def display_configs(config_key, config_value):
    if isinstance(config_value, list):
        print(f"[{BLUE}] {config_key}:")
        for config_dict in config_value:
            print(f"[{BLUE}]   {'-'*5}")
            for key in config_dict:
                display_configs('  '+key, config_dict.get(key))
    else:
        print(f"[{BLUE}] {config_key}[{GREEN}]: {config_value}")
    



from rich.prompt import Confirm
from cmdcheatsheet.alt_store.core import add_alt_store, is_existing_store_name, update_alt_store
from cmdcheatsheet.shared.messages import show_invalid_store_location_message
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails
from cmdcheatsheet.alt_store.models import AlternativeStore
from cmdcheatsheet.config.validators import is_valid_custom_commands_location


class AddAltStore(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--add-alternative-store', '-aas'],
        "Add alternative commands store (JSON file) location.",
        [CommandArgument('store_name'), CommandArgument('store_location')])

    def handler(self, args):
        alt_store = AlternativeStore(args[0], args[1])
        if not is_valid_custom_commands_location(alt_store.location):
            show_invalid_store_location_message()
        elif is_existing_store_name(alt_store.name):
            is_yes = Confirm.ask(
                f"Store with the name '{alt_store.name}' already exists. " +
                "Do you want to override an existing one?")
            if is_yes:
                update_alt_store(alt_store)
        else: 
            add_alt_store(alt_store)




from cmdcheatsheet.general.consts import HELP_RESPONSIBLE_COMMANDS
from cmdcheatsheet.general.display import display_help


def help(command_list):
    for c in command_list:
        commands_to_help = []
        for responsible_command in c.responsible_commands:
            command_arguments = [f'<{a.name}{":optional" if not a.required else ""}>' for a in c.command_arguments]
            arguments = (' '  + ' '.join(command_arguments)) if command_arguments else ''
            commands_to_help.append(f'{responsible_command}{arguments}')
        display_help(', '.join(commands_to_help), c.description)

def get_command_by_name(program_actions, command_name):
    return next((c for c in program_actions if c.need_to_handle(command_name)), None)

def is_help_action(command):
    return command in HELP_RESPONSIBLE_COMMANDS



from cmdcheatsheet.commands.commands import Commands
from cmdcheatsheet.commands.detailed_commands import DetailedCommands
from cmdcheatsheet.commands.table_view_commands import TableViewCommands
from cmdcheatsheet.commands.add_command import AddCommand
from cmdcheatsheet.commands.update_command import UpdateCommand
from cmdcheatsheet.commands.delete_command import DeleteCommand
from cmdcheatsheet.commands.find_command import FindCommand
from cmdcheatsheet.commands.detailed_find_command import DetailedFindCommand
from cmdcheatsheet.commands.table_view_find_command import TableViewFindCommand
from cmdcheatsheet.commands.available_command_names import AvailableCommandNames

command_actions = [
    Commands(),
    DetailedCommands(),
    TableViewCommands(),
    AddCommand(),
    UpdateCommand(),
    DeleteCommand(),
    FindCommand(),
    DetailedFindCommand(),
    TableViewFindCommand(),
    AvailableCommandNames()
]



from rich import print
from cmdcheatsheet.config.consts import ALT_STORE_LOCATION_CONF, ALT_STORE_NAME_CONF
from cmdcheatsheet.shared.display import BLUE, GREEN


def display_alt_store_list(alt_stores):
    for store in alt_stores:
        display_alt_store(store.get(ALT_STORE_NAME_CONF), store.get(ALT_STORE_LOCATION_CONF))

def display_alt_store(store_name, store_value):
    print(f"[{BLUE}]  {store_name}[{GREEN}] - {store_value}")




from rich import print
from cmdcheatsheet.alt_store.core import get_applied_alt_store_name
from cmdcheatsheet.shared.models import CommandDetails
from cmdcheatsheet.shared.display import YELLOW, BLUE


class DisplayAppliedAltStoreName(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--display-applied-alternative-store-name', '-daasn'],
        "Display the name of applied alternative store.")

    def handler(self, _):
        applied_store_name = get_applied_alt_store_name()
        if applied_store_name: 
            print(f"[{BLUE}]{applied_store_name}")
        else:
            print(f"[{YELLOW}]None of the alternative stores were applied.")



from cmdcheatsheet.commands.display import display_command_list
from cmdcheatsheet.shared.models import CommandDetails


class Commands(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--commands', '-c'],
            'Display all commands.')

    def handler(self, _):
        display_command_list()

